#include <SPI.h>
#include <Adb.h>
#include <Servo.h>
#include <EEPROM.h>

// 状態定義
#define IDLE  0
#define IR_RECEIVE_MODE  1
#define IR_SEND_MODE  2
#define STATE_ACK  3
#define USB_POW_ON  4
#define USB_POW_OFF  5
#define IR_RESO  2


// コマンド定義
#define SEND_H	      0xFF
#define SEND_L	      0xFD

#define RECEIVE_H    0xFF
#define RECEIVE_L    0xFE

#define USB_H  0xFF
#define USB_L_ON  0xFB
#define USB_L_OFF  0xFA

// ピン定義
// ディジタルピンであればどのピンでも構いません
int ir_in = 2;
int ir_out = 3;
int usb_pow = 4;

// 状態
int state = IDLE;

// チャンネル
int ch = 1;

// ACK
byte ack_s[2] = {0xFF,0x1F};
byte ack_f[2] = {0xFF,0x2F};

// 学習と送信時に使用するデータバッファ
byte ir_buffer[512] = {};

unsigned short buffer[256];
unsigned short data_length = 0;

// データ長さ検出用
unsigned long us = micros();

// Adb connection.
Connection * connection;

void setup()
{

  // Initialise serial port
  Serial.begin(57600);
  ADB::init();
  // Open an ADB stream to the phone's shell. Auto-reconnect
  connection = ADB::addConnection("tcp:4567", true, adbEventHandler);
  // 赤外線入出力ピンの設定
  pinMode(ir_in, INPUT);
  pinMode(ir_out, OUTPUT);
  // LED接続ピンの出力設定
  pinMode(13, OUTPUT);
  // USB電源タップ接続ピン出力設定
  pinMode(usb_pow,OUTPUT);
  digitalWrite(usb_pow,0);
  Serial.println("START IR REMOTE PROGRAM"); 
}

void loop()
{
  int rtn = 0;
  while(1)
  {

    if(state == IR_RECEIVE_MODE)
    {
      //LedBlink(3,150);
      rtn = ir_receive(ch);
      if(rtn == 1)
      {
      	// 受信成功をAndroidに通知
        while(connection->write(2, (uint8_t*)&ack_s) < 0) ADB::poll();
        Serial.println("receive : OK");
      }
      else
      {
      	// 受信失敗をAndroidに通知
        while(connection->write(2, (uint8_t*)&ack_f) < 0) ADB::poll();
        Serial.println("receive : NG");
      }
      state = IDLE;
    }
    else if(state == IR_SEND_MODE)
    {
      //LedBlink(5,50);
      ir_send(ch);
      state = IDLE;
    }
    else if(state == USB_POW_ON)
    {
      digitalWrite(usb_pow,1);
      state = IDLE;  
    }
    else if(state == USB_POW_OFF)
    {
      digitalWrite(usb_pow,0);
      state = IDLE;
    }
    else
    {
      // 待機
      ADB::poll();
    }
  }
}

byte ir_receive(int ch)
{
  int count = 0;
  int adress = 0;
  byte val = 0;
  byte before_val = 1;
  unsigned short time = 0;
  unsigned int cnt = 0;
  int receive_loop = 1;
  int data_count = 0;
//  Serial.println("----- IR RECEIVE MODE -----");
  while(receive_loop)
  {
    while( (val = digitalRead(ir_in)) == before_val ) 
    { 
      if(time != 0)
      {
        cnt++;
      }
      if(cnt > 20000)
      {
        receive_loop = 0;
        Serial.println("break");
        break;
      }
    }
    unsigned long us2 = micros();
    time =((us2-us)/IR_RESO);
    ir_buffer[data_count] = (byte)((time&0xFF00) >> 8);
    ir_buffer[data_count+1] = (byte)(time & 0x00FF);
    data_count+=2;
    before_val = val;
    us = us2;
    cnt = 0;
  }
  // dataが1024を超えたら失敗,データ数10以下のノイズなら失敗とする
  if(data_count > 512 || data_count < 10)
  {
//    Serial.println("----- ERROR -----");
    return 0;
  }
  else
  {
    adress = 512*(ch-1);  
    /*
    // 受信データをモニタに表示する
    for(cnt = 0; cnt < data_count; cnt+=2)
    {
      Serial.print(((unsigned short)(ir_buffer[cnt]<<8) +  ir_buffer[cnt+1]),DEC);
      Serial.print(",");
    }
    */
    
    for(cnt = 0; cnt < data_count; cnt++)
    {
      EEPROM.write(adress + cnt,ir_buffer[cnt]);
    }
//    Serial.println("Done EEPROM Write");
    
    // 終端として0を代入
    EEPROM.write(adress + data_count,0x00);
    EEPROM.write(adress + data_count+1,0x00);
    
    Serial.print("Signal Length : ");
    Serial.println(data_count/2,DEC);
//    Serial.println("----- RECEIVE DONE -----");
    
    return 1;
  }
}

void ir_send(int ch)
{
//  Serial.println("----- IR SEDN MODE -----");
  int cnt = 0;
  int i = 0;
  int adress = 512 * (ch-1);
  for(i = 2; i < 512; i+=2)
  {
    buffer[cnt] = ((EEPROM.read(adress + i)<<8) + EEPROM.read(adress + i + 1));
    if(buffer[cnt] == 0) break;
    // 送信データをモニタを表示
    //Serial.print(buffer[cnt],DEC);
    //Serial.print(",");
    cnt++;
  }
  for (int cnt = 0; cnt < 256; cnt++) {
    // (us/IR_RESO)の分解能でON/OFF時間を記録されている
    unsigned long len = buffer[cnt]*IR_RESO;
    if (len == 0) break;
    unsigned long us = micros();
    do {
      // 交互にON/OFFを繰り返す
      digitalWrite(ir_out, 1 - (cnt&1));
      // 38kHz発振にするように時間を調整
      delayMicroseconds(8);
      digitalWrite(ir_out, 0);
      delayMicroseconds(7);
    } 
    while (long(us + len - micros()) > 0); // 送信時間に達するまでループ
  }
//  Serial.println("----- SEND DONE ------");
  
}

// LED点滅関数
/*void LedBlink(int num,int ms)
{
  int i = 0;
  for(i =0; i < num; i++)
  {
    digitalWrite(13, HIGH);
    delay(ms);
    digitalWrite(13, LOW);
    delay(ms);  
  } 
}*/

// Event handler for the shell connection. 
void adbEventHandler(Connection * connection, adb_eventType event, uint16_t length, uint8_t * rec_data)
{
  // Androidからデータを受信した場合の処理
  if (event == ADB_CONNECTION_RECEIVE)
  {
    // IR_RECEIVE_MODE (0xFF,0xFE,ch)
    if(rec_data[0]== RECEIVE_H && rec_data[1] == RECEIVE_L)
    {
      ch = rec_data[2];
      state = IR_RECEIVE_MODE;
      Serial.print("RECEIVE CH : ");
      Serial.println(ch,DEC);
    }
    // IR_SEND_MODE (0xFE,0xFD,ch)
    else if(rec_data[0]== SEND_H && rec_data[1] == SEND_L)
    {
      ch = rec_data[2];
      state = IR_SEND_MODE;
      Serial.print("SEND CH : ");
      Serial.println(ch,DEC);
    }
    else if(rec_data[0]== USB_H && rec_data[1] == USB_L_ON)
    {
      state = USB_POW_ON;
    }
    else if(rec_data[0]== USB_H && rec_data[1] == USB_L_OFF)
    {
      state = USB_POW_OFF;
    }
    else
    {
      state = IDLE;
    }
  }
}
